@@ -1,4 +1,6 @@
 # ggplot2 convenience functions for internal use --------------------------
+
+
 facet_wrap_parsed <- function(...) {
   facet_wrap(..., labeller = label_parsed)
 }
@@ -12,7 +14,7 @@ dont_expand_axes <- function() {
   coord_cartesian(expand = FALSE)
 }
 force_axes_in_facets <- function() {
-  thm <- theme_default()
+  thm <- ggplot2::theme_get()
   annotate("segment",
            x = c(-Inf, -Inf), xend = c(Inf,-Inf),
            y = c(-Inf,-Inf), yend = c(-Inf, Inf),
@@ -28,3 +30,12 @@ reduce_legend_spacing <- function(cm) {
 space_legend_keys <- function(relative_size = 2, color = "white") {
   theme(legend.key = element_rect(size = rel(relative_size), color = color))
 }
+
+
+# set aesthetic mapping for histograms depending on freq argument
+set_hist_aes <- function(freq = TRUE, ...) {
+  if (freq)
+    aes_(x = ~ value, ...)
+  else
+    aes_(x = ~ value, y = ~ ..density.., ...)
+}
