@@ -14,7 +14,7 @@
 #'   \code{\link[ggplot2]{geom_line}}'s default size (for \code{mcmc_acf}).
 #' @param ... Currently ignored.
 #'
-#' @template return-ggplot
+#' @template return-ggplot-or-data
 #'
 #' @section Plot Descriptions:
 #' \describe{
@@ -125,47 +125,42 @@ NULL
 
 
 # Rhat --------------------------------------------------------------------
+
 #' @rdname MCMC-diagnostics
 #' @export
 #' @param rhat A vector of \code{\link[=rhat]{Rhat}} estimates.
 #'
 mcmc_rhat <- function(rhat, ..., size = NULL) {
   check_ignored_arguments(...)
-  rhat <- validate_rhat(rhat)
-  plot_data <- diagnostic_data_frame(
-    x = rhat,
-    diagnostic = "rhat"
-  )
+  data <- mcmc_rhat_data(rhat)
+
   graph <- ggplot(
-    data = plot_data,
+    data = data,
     mapping = aes_(
       x = ~ value,
-      y = ~ factor_by_name,
-      color = ~ factor_by_value,
-      fill = ~ factor_by_value
-    )
-  ) +
+      y = ~ parameter,
+      color = ~ rating,
+      fill = ~ rating)) +
     geom_segment(
       mapping = aes_(
-        yend = ~ factor_by_name,
-        xend = ifelse(min(rhat) < 1, 1, -Inf)
-      ),
-      na.rm = TRUE
-    )
+        yend = ~ parameter,
+        xend = ifelse(min(data$value) < 1, 1, -Inf)),
+      na.rm = TRUE)
 
-  if (min(rhat) < 1)
+  if (min(data$value) < 1) {
     graph <- graph +
       vline_at(1, color = "gray", size = 1)
+  }
+
+  brks <- set_rhat_breaks(data$value)
 
-  brks <- set_rhat_breaks(rhat)
   graph +
     diagnostic_points(size) +
     vline_at(
       brks[-1],
       color = "gray",
       linetype = 2,
-      size = 0.25
-    ) +
+      size = 0.25) +
     labs(y = NULL, x = expression(hat(R))) +
     scale_fill_diagnostic("rhat") +
     scale_color_diagnostic("rhat") +
@@ -180,22 +175,18 @@ mcmc_rhat <- function(rhat, ..., size = NULL) {
 #' @export
 mcmc_rhat_hist <- function(rhat, ..., binwidth = NULL) {
   check_ignored_arguments(...)
+  data <- mcmc_rhat_data(rhat)
+
   ggplot(
-    data = diagnostic_data_frame(
-      x = validate_rhat(rhat),
-      diagnostic = "rhat"
-    ),
+    data = data,
     mapping = aes_(
       x = ~ value,
-      color = ~ factor_by_value,
-      fill = ~ factor_by_value
-    )
-  ) +
+      color = ~ rating,
+      fill = ~ rating)) +
     geom_histogram(
       size = .25,
       na.rm = TRUE,
-      binwidth = binwidth
-    ) +
+      binwidth = binwidth) +
     scale_color_diagnostic("rhat") +
     scale_fill_diagnostic("rhat") +
     labs(x = expression(hat(R)), y = NULL) +
@@ -205,8 +196,18 @@ mcmc_rhat_hist <- function(rhat, ..., binwidth = NULL) {
     yaxis_ticks(FALSE)
 }
 
+#' @rdname MCMC-diagnostics
+#' @export
+mcmc_rhat_data <- function(rhat, ...) {
+  check_ignored_arguments(...)
+  diagnostic_data_frame(
+    x = validate_rhat(rhat),
+    diagnostic = "rhat")
+}
+
 
 # effective sample size ---------------------------------------------------
+
 #' @rdname MCMC-diagnostics
 #' @export
 #' @param ratio A vector of \emph{ratios} of effective sample size estimates to
@@ -214,29 +215,24 @@ mcmc_rhat_hist <- function(rhat, ..., binwidth = NULL) {
 #'
 mcmc_neff <- function(ratio, ..., size = NULL) {
   check_ignored_arguments(...)
+  data <- mcmc_neff_data(ratio)
+
   ggplot(
-    data = diagnostic_data_frame(
-      x = validate_neff_ratio(ratio),
-      diagnostic = "neff"
-    ),
+    data,
     mapping = aes_(
       x = ~ value,
-      y = ~ factor_by_name,
-      color = ~ factor_by_value,
-      fill = ~ factor_by_value
-    )
-  ) +
+      y = ~ parameter,
+      color = ~ rating,
+      fill = ~ rating)) +
     geom_segment(
-      aes_(yend = ~factor_by_name, xend = -Inf),
-      na.rm = TRUE
-    ) +
+      aes_(yend = ~ parameter, xend = -Inf),
+      na.rm = TRUE) +
     diagnostic_points(size) +
     vline_at(
       c(0.1, 0.5, 1),
       color = "gray",
       linetype = 2,
-      size = 0.25
-    ) +
+      size = 0.25) +
     labs(y = NULL, x = expression(N[eff]/N)) +
     scale_fill_diagnostic("neff") +
     scale_color_diagnostic("neff") +
@@ -244,8 +240,7 @@ mcmc_neff <- function(ratio, ..., size = NULL) {
       breaks = c(0, 0.1, 0.25, 0.5, 0.75, 1),
       labels = c("0", "0.1", "0.25", "0.5", "0.75", "1"),
       limits = c(0, 1.05),
-      expand = c(0, 0)
-    ) +
+      expand = c(0, 0)) +
     yaxis_text(FALSE) +
     yaxis_title(FALSE) +
     yaxis_ticks(FALSE)
@@ -255,22 +250,18 @@ mcmc_neff <- function(ratio, ..., size = NULL) {
 #' @export
 mcmc_neff_hist <- function(ratio, ..., binwidth = NULL) {
   check_ignored_arguments(...)
+  data <- mcmc_neff_data(ratio)
+
   ggplot(
-    data = diagnostic_data_frame(
-      x = validate_neff_ratio(ratio),
-      diagnostic = "neff"
-    ),
+    data,
     mapping = aes_(
       x = ~ value,
-      color = ~ factor_by_value,
-      fill = ~ factor_by_value
-    )
-  ) +
+      color = ~ rating,
+      fill = ~ rating)) +
     geom_histogram(
       size = .25,
       na.rm = TRUE,
-      binwidth = binwidth
-    ) +
+      binwidth = binwidth) +
     scale_color_diagnostic("neff") +
     scale_fill_diagnostic("neff") +
     labs(x = expression(N[eff]/N), y = NULL) +
@@ -280,6 +271,14 @@ mcmc_neff_hist <- function(ratio, ..., binwidth = NULL) {
     yaxis_ticks(FALSE)
 }
 
+#' @rdname MCMC-diagnostics
+#' @export
+mcmc_neff_data <- function(ratio, ...) {
+  check_ignored_arguments(...)
+  diagnostic_data_frame(
+    x = validate_neff_ratio(ratio),
+    diagnostic = "neff")
+}
 
 # autocorrelation ---------------------------------------------------------
 #' @rdname MCMC-diagnostics
@@ -287,10 +286,8 @@ mcmc_neff_hist <- function(ratio, ..., binwidth = NULL) {
 #' @template args-mcmc-x
 #' @template args-pars
 #' @template args-regex_pars
-#' @param facet_args Arguments (other than \code{facets}) passed to
-#'   \code{\link[ggplot2]{facet_grid}} to control faceting.
+#' @template args-facet_args
 #' @param lags The number of lags to show in the autocorrelation plot.
-#'
 mcmc_acf <-
   function(x,
            pars = character(),
@@ -339,42 +336,56 @@ diagnostic_points <- function(size = NULL) {
 
 # @param x The object returned by validate_rhat or validate_neff_ratio
 diagnostic_data_frame <- function(x, diagnostic = c("rhat", "neff")) {
-  diagnostic <- match.arg(diagnostic)
-  fac <- if (!is.null(names(x))) {
-    factor(x, labels = names(sort(x)))
-  } else {
-    factor(x)
-  }
+  x <- auto_name(sort(x))
+  stopifnot(!anyDuplicated(names(x)))
 
+  diagnostic <- match.arg(diagnostic)
   fun <- match.fun(paste0("factor_", diagnostic))
-  d <- data.frame(
+  d <- dplyr::data_frame(
+    diagnostic = diagnostic,
+    parameter = factor(seq_along(x), labels = names(x)),
     value = x,
-    factor_by_name = fac,
-    factor_by_value = factor(fun(x), levels = c("high", "ok", "low"))
+    rating = factor(fun(x), levels = c("high", "ok", "low"))
   )
-  # d$factor_by_value <- factor(d$factor_by_value, levels = c("high", "ok", "low"))
+
+  labels <- diagnostic_color_labels[[diagnostic]]
+  d$description <- as.character(labels[d$rating])
+
   rownames(d) <- NULL
-  return(d)
+  d
 }
 
-# Convert numeric vector of Rhat values to a factor
-#
-# @param x A numeric vector
-# @param breaks A numeric vector of length two. The resulting factor variable
-#   will have three levels ('low', 'ok', and 'high') corresponding to (x <=
-#   breaks[1], breaks[1] < x <= breaks[2], x > breaks[2]).
-# @return A factor the same length as x with three levels.
-#
+
+auto_name <- function(xs) {
+  if (is.null(names(xs))) {
+    names(xs) <- zero_pad_int(seq_along(xs))
+  }
+  xs
+}
+
+# c(1, 2, 10, 20, 100) => c("001", "002", "010", "020", "100")
+zero_pad_int <- function(xs) {
+  formatter <- paste0("%0", max(nchar(xs)), "d")
+  sprintf(formatter, xs)
+}
+
+
+#' Convert numeric vector of Rhat values to a factor
+#'
+#' @param x A numeric vector
+#' @param breaks A numeric vector of length two. The resulting factor variable
+#'   will have three levels ('low', 'ok', and 'high') corresponding to (x <=
+#'   breaks[1], breaks[1] < x <= breaks[2], x > breaks[2]).
+#' @return A factor the same length as x with three levels.
+#' @noRd
 factor_rhat <- function(x, breaks = c(1.05, 1.1)) {
   stopifnot(is.numeric(x),
             isTRUE(all(x > 0)),
             length(breaks) == 2)
-  cut(
-    x,
-    breaks = c(-Inf, breaks, Inf),
-    labels = c("low", "ok", "high"),
-    ordered_result = FALSE
-  )
+  cut(x,
+      breaks = c(-Inf, breaks, Inf),
+      labels = c("low", "ok", "high"),
+      ordered_result = FALSE)
 }
 
 # factor neff ratio
@@ -388,6 +399,7 @@ scale_color_diagnostic <- function(diagnostic = c("rhat", "neff")) {
   d <- match.arg(diagnostic)
   diagnostic_color_scale(d, aesthetic = "color")
 }
+
 scale_fill_diagnostic <- function(diagnostic = c("rhat", "neff")) {
   d <- match.arg(diagnostic)
   diagnostic_color_scale(d, aesthetic = "fill")
@@ -398,24 +410,14 @@ diagnostic_color_scale <- function(diagnostic = c("rhat", "neff"),
   diagnostic <- match.arg(diagnostic)
   aesthetic <- match.arg(aesthetic)
   color_levels <- c("light", "mid", "dark")
-  if (diagnostic == "neff")
+  if (diagnostic == "neff") {
     color_levels <- rev(color_levels)
-  if (aesthetic == "color")
-    color_levels <- paste0(color_levels, "_highlight")
-
-  color_labels <- if (diagnostic == "rhat") {
-    c(
-      expression(hat(R) > 1.10),
-      expression(hat(R) <= 1.10),
-      expression(hat(R) <= 1.05)
-    )
-  } else {
-    c(
-      expression(N[eff]/N > 0.5),
-      expression(N[eff]/N <= 0.5),
-      expression(N[eff]/N <= 0.1)
-    )
   }
+  if (aesthetic == "color") {
+    color_levels <- paste0(color_levels, "_highlight")
+  }
+
+  color_labels <- diagnostic_color_labels[[diagnostic]]
 
   do.call(
     match.fun(paste0("scale_", aesthetic, "_manual")),
@@ -428,42 +430,57 @@ diagnostic_color_scale <- function(diagnostic = c("rhat", "neff"),
   )
 }
 
+diagnostic_color_labels <- list(
+  rhat = c(
+    high = expression(hat(R) > 1.10),
+    ok   = expression(hat(R) <= 1.10),
+    low  = expression(hat(R) <= 1.05)
+  ),
+  neff = c(
+    high = expression(N[eff] / N > 0.5),
+    ok   = expression(N[eff] / N <= 0.5),
+    low  = expression(N[eff] / N <= 0.1)
+  )
+)
+
 # set x-axis breaks based on rhat values
 set_rhat_breaks <- function(rhat) {
   br <- c(1, 1.05)
-  if (any(rhat > 1.05))
+  if (any(rhat > 1.05)) {
     br <- c(br, 1.1)
-  for (k in c(1.5, 2)) {
-    if (any(rhat > k))
-      br <- c(br, k)
   }
-  if (max(rhat) >= max(br) + .1)
-    br <- c(br, round(max(rhat), 2))
 
-  return(br)
+  for (k in c(1.5, 2)) {
+    if (any(rhat > k)) {
+      br <- c(br, k)
+    }
+  }
+  if (max(rhat) >= max(br) + .1) {
+    br <- c(br, round(max(rhat), 2))
+  }
+  br
 }
 
 
 
 # drop NAs from a vector and issue warning
 drop_NAs_and_warn <- function(x) {
-  if (!anyNA(x))
-    return(x)
-
   is_NA <- is.na(x)
-  warning(
-    "Dropped ", sum(is_NA), " NAs from '",
-    deparse(substitute(x)), "'."
-  )
+  if (anyNA(x)) {
+    warning(
+      "Dropped ", sum(is_NA), " NAs from '",
+      deparse(substitute(x)), "'."
+    )
+  }
   x[!is_NA]
 }
 
 # either throws error or returns an rhat vector (dropping NAs)
 validate_rhat <- function(rhat) {
   stopifnot(is_vector_or_1Darray(rhat))
-  if (any(rhat < 0, na.rm = TRUE))
+  if (any(rhat < 0, na.rm = TRUE)) {
     stop("All 'rhat' values must be positive.")
-
+  }
   rhat <- setNames(as.vector(rhat), names(rhat))
   drop_NAs_and_warn(rhat)
 }
@@ -471,9 +488,9 @@ validate_rhat <- function(rhat) {
 # either throws error or returns as.vector(ratio)
 validate_neff_ratio <- function(ratio) {
   stopifnot(is_vector_or_1Darray(ratio))
-  if (any(ratio < 0 | ratio > 1, na.rm = TRUE))
+  if (any(ratio < 0 | ratio > 1, na.rm = TRUE)) {
     stop("All elements of 'ratio' must be between 0 and 1.")
-
+  }
   ratio <- setNames(as.vector(ratio), names(ratio))
   drop_NAs_and_warn(ratio)
 }
