@@ -19,6 +19,17 @@
 #'   \emph{length two} passed to \code{\link[ggplot2]{geom_hex}} to override the
 #'   default binwidth in both the vertical and horizontal directions.
 #'
+#' @param np Optionally, a data frame of NUTS sampler parameters, either created
+#'   by \code{\link{nuts_params}} or in the same form as the object returned by
+#'   \code{\link{nuts_params}}. The colors, shapes, and sizes of the
+#'   superimposed points can be customized using the \code{np_style} argument.
+#' @param np_style If \code{np} is specified, \code{np_style} can be a call to
+#'   the \code{scatter_style_np} helper function (for \code{mcmc_scatter}) or
+#'   the \code{pairs_style_np} helper function (for \code{mcmc_pars}) to specify
+#'   arguments controlling the appearance of superimposed points representing
+#'   NUTS diagnostic information. (Note: for \code{pairs_style_np} the
+#'   \code{size} arguments are interpreted as scaling factors).
+#'
 #' @return \code{mcmc_scatter} and \code{mcmc_hex} return a ggplot object that
 #'   can be further customized using the \pkg{ggplot2} package.
 #'
@@ -30,7 +41,9 @@
 #'   \item{\code{mcmc_scatter}}{
 #'    Bivariate scatterplot of posterior draws. If using a very large number of
 #'    posterior draws then \code{mcmc_hex} may be preferable to avoid
-#'    overplotting.
+#'    overplotting. For models fit using \code{\link{NUTS}} the \code{np},
+#'    and \code{np_style} arguments can be used to add additional information in
+#'    the plot (in this case the approximate location of divergences).
 #'   }
 #'   \item{\code{mcmc_hex}}{
 #'    Hexagonal heatmap of 2-D bin counts. This plot is useful in cases where
@@ -46,13 +59,27 @@
 #'    (roughly) half are displayed above the diagonal and half are below (all
 #'    chains are always merged together for the plots along the diagonal). Other
 #'    possibilities are available by setting the \code{condition} argument.
+#'
 #'    Additionally, extra diagnostic information for models fit using
 #'    \code{\link{NUTS}} can be added to the pairs plot using the \code{lp},
-#'    \code{np}, and \code{np_style} arguments.
+#'    \code{np}, and \code{np_style} arguments. If \code{np} is specified (and
+#'    \code{condition} is \emph{not} \code{"divergent__"}), then points (red, by
+#'    default) will be superimposed onto the off-diagonal plots indicating which
+#'    (if any) iterations encountered a divergent transition. Also, if both
+#'    \code{np} and \code{max_treedepth} are specified then points (yellow, by
+#'    default) will be superimposed to indicate a transition that hit the
+#'    maximum treedepth rather than terminated its evolution normally. The
+#'    \code{np_style} argument can be used with the \code{pairs_style_np}
+#'    convenience function to change the appearance of these overlaid points.
+#'    See the \strong{Examples} section.
 #'   }
 #' }
 #'
+#' @template reference-vis-paper
+#'
 #' @examples
+#' library("ggplot2")
+#'
 #' # some parameter draws to use for demonstration
 #' x <- example_mcmc_draws(params = 6)
 #' dimnames(x)
@@ -62,7 +89,7 @@
 #' (p <- mcmc_scatter(x, pars = c("alpha", "sigma"),
 #'                   transform = list(sigma = "log")))
 #' p +
-#'   ggplot2::labs(
+#'   labs(
 #'     title = "Insert your own headline-grabbing title",
 #'     subtitle = "with a provocative subtitle",
 #'     caption = "and a controversial caption",
@@ -71,18 +98,18 @@
 #'    )
 #'
 #' # add ellipse
-#' p + ggplot2::stat_ellipse(level = 0.9, color = "gray20", size = 1)
+#' p + stat_ellipse(level = 0.9, color = "gray20", size = 1)
 #'
 #' # add contour
 #' color_scheme_set("red")
 #' p2 <- mcmc_scatter(x, pars = c("alpha", "sigma"), size = 3.5, alpha = 0.25)
-#' p2 + ggplot2::stat_density_2d(color = "black", size = .5)
+#' p2 + stat_density_2d(color = "black", size = .5)
 #'
 #' # can also add lines/smooths
 #' color_scheme_set("pink")
 #' (p3 <- mcmc_scatter(x, pars = c("alpha", "beta[3]"), alpha = 0.25, size = 3))
-#' p3 + ggplot2::geom_smooth(method = "lm", se = FALSE, color = "gray20",
-#'                           size = .75, linetype = 2)
+#' p3 + geom_smooth(method = "lm", se = FALSE, color = "gray20",
+#'                  size = .75, linetype = 2)
 #'
 #' \donttest{
 #' # hexagonal heatmap
@@ -101,7 +128,9 @@ mcmc_scatter <- function(x,
                          transformations = list(),
                          ...,
                          size = 2.5,
-                         alpha = 0.8) {
+                         alpha = 0.8,
+                         np = NULL,
+                         np_style = scatter_style_np()) {
   check_ignored_arguments(...)
   .mcmc_scatter(
     x,
@@ -111,7 +140,9 @@ mcmc_scatter <- function(x,
     size = size,
     alpha = alpha,
     hex = FALSE,
-    binwidth = NULL
+    binwidth = NULL,
+    np = np,
+    np_style = np_style
   )
 }
 
@@ -158,24 +189,6 @@ mcmc_hex <- function(x,
 #'   up to a constant. \code{lp} should either be created via
 #'   \code{\link{log_posterior}} or be an object with the same form as the
 #'   object returned by \code{\link{log_posterior}}.
-#' @param np For \code{mcmc_pairs}, a molten data frame of NUTS sampler
-#'   parameters, either created by \code{\link{nuts_params}} or in the same form
-#'   as the object returned by \code{\link{nuts_params}}. If \code{np} is
-#'   specified (and \code{condition} is \emph{not} \code{"divergent__"}), then
-#'   points (red, by default) will be superimposed onto the off-diagonal plots
-#'   indicating which (if any) iterations encountered a divergent transition.
-#'   Also, if both \code{np} and \code{max_treedepth} are specified then points
-#'   (yellow, by default) will be superimposed to indicate a transition that hit
-#'   the maximum treedepth rather than terminated its evolution normally. The
-#'   colors, shapes, and sizes of the superimposed points can be customized
-#'   using the \code{np_style} argument.
-#' @param np_style For \code{mcmc_pairs}, a call to the \code{pairs_style_np}
-#'   helper function to specify arguments controlling the appearance of
-#'   superimposed points representing NUTS diagnostic parameter warnings (if the
-#'   \code{np} argument is specified). The arguments to \code{pairs_style_np}
-#'   correspond to setting the color, shape, and size of the points indicating
-#'   divergences and the points indicating hitting the maximum treedepth (Note:
-#'   here "size" is interpreted as a scaling factor).
 #' @param max_treedepth For \code{mcmc_pairs}, an integer representing the
 #'   maximum treedepth allowed when fitting the model (if fit using NUTS). This
 #'   is only needed for detecting which transitions (if any) hit the maximum
@@ -214,8 +227,9 @@ mcmc_hex <- function(x,
 #' }
 #'
 #' \dontrun{
-#' # pairs plot with NUTS diagnostic info overlaid,
-#' # example using rstanarm package
+#' ### Adding NUTS diagnostics to scatterplots and pairs plots
+#'
+#' # examples using rstanarm package
 #' library(rstanarm)
 #'
 #' # for demonstration purposes, intentionally fit a model that
@@ -230,8 +244,18 @@ mcmc_hex <- function(x,
 #' posterior <- as.array(fit)
 #' np <- nuts_params(fit)
 #'
+#' # mcmc_scatter with divergences highlighted
+#' color_scheme_set("brightblue")
+#' mcmc_scatter(posterior, pars = c("wt", "sigma"), np = np)
+#'
+#' color_scheme_set("darkgray")
+#' div_style <- scatter_style_np(div_color = "green", div_shape = 4, div_size = 4)
+#' mcmc_scatter(posterior, pars = c("sigma", "(Intercept)"),
+#'              np = np, np_style = div_style)
+#'
 #' # split the draws according to above/below median accept_stat__ and
 #' # show approximate location of divergences (red points)
+#' color_scheme_set("brightblue")
 #' mcmc_pairs(
 #'   posterior,
 #'   pars = c("wt", "cyl", "sigma"),
@@ -282,7 +306,7 @@ mcmc_pairs <- function(x,
   stopifnot(
     is.list(diag_args),
     is.list(off_diag_args),
-    inherits(np_style, "pairs_style_np"),
+    inherits(np_style, "nuts_style"),
     inherits(condition, "pairs_condition")
   )
   plot_diagonal <- pairs_plotfun(match.arg(diag_fun))
@@ -305,11 +329,13 @@ mcmc_pairs <- function(x,
   no_lp <- is.null(lp)
   no_max_td <- is.null(max_treedepth)
   if (!no_np) {
+    param <- sym("Parameter")
+    val <- sym("Value")
     np <- validate_nuts_data_frame(np, lp)
-    divs <- filter_(np, ~ Parameter == "divergent__")$Value
+    divs <- dplyr::filter(np, UQ(param) == "divergent__") %>% pull(UQ(val))
     divergent__ <- matrix(divs, nrow = n_iter * n_chain, ncol = n_param)[, 1]
     if (!no_max_td) {
-      gt_max_td <- filter_(np, ~ Parameter == "treedepth__")$Value > max_treedepth
+      gt_max_td <- (dplyr::filter(np, UQ(param) == "treedepth__") %>% pull(UQ(val))) > max_treedepth
       max_td_hit__ <- matrix(gt_max_td, nrow = n_iter * n_chain, ncol = n_param)[, 1]
     }
   }
@@ -363,6 +389,7 @@ mcmc_pairs <- function(x,
           geom_point(
             aes_(color = divs_j_fac, size = divs_j_fac),
             shape = np_style$shape[["div"]],
+            alpha = np_style$alpha[["div"]],
             na.rm = TRUE
           )
       }
@@ -373,6 +400,7 @@ mcmc_pairs <- function(x,
           geom_point(
             aes_(color = max_td_hit_j_fac, size = max_td_hit_j_fac),
             shape = np_style$shape[["td"]],
+            alpha = np_style$alpha[["td"]],
             na.rm = TRUE
           )
       }
@@ -391,36 +419,62 @@ mcmc_pairs <- function(x,
 
 #' @rdname MCMC-scatterplots
 #' @export
-#' @param div_color,div_shape,div_size,td_color,td_shape,td_size Optional
-#'   arguments to the \code{pairs_style_np} helper function that are eventually
-#'   passed to \code{\link[ggplot2]{geom_point}}. They control the color, shape,
-#'   and size specifications  for points representing divergences (\code{div})
-#'   and points indicating hitting the maximum treedepth (\code{td}). See the
-#'   \code{np_style} argument for more details. The default values are displayed
-#'   in the \strong{Usage} section above.
+#' @param div_color,div_shape,div_size,div_alpha,td_color,td_shape,td_size,td_alpha
+#'   Optional arguments to the \code{scatter_style_np} or \code{pairs_style_np}
+#'   helper functions that are eventually passed to
+#'   \code{\link[ggplot2]{geom_point}}.The default values are displayed in the
+#'   \strong{Usage} section above.
+scatter_style_np <-
+  function(div_color = "red",
+           div_shape = 16,
+           div_size = 2.5,
+           div_alpha = 1) {
+    stopifnot(
+      is.numeric(div_shape) || is.character(div_shape),
+      is.character(div_color),
+      is.numeric(div_size),
+      is.numeric(div_alpha) && div_alpha >= 0 && div_alpha <= 1
+    )
+    style <- list(
+      color = c(div = div_color),
+      shape = c(div = div_shape),
+      size = c(div = div_size),
+      alpha = c(div = div_alpha)
+    )
+    structure(style, class = c(class(style), "nuts_style"))
+  }
+
+#' @rdname MCMC-scatterplots
+#' @export
 pairs_style_np <-
   function(div_color = "red",
            div_shape = 4,
            div_size = 1,
+           div_alpha = 1,
            td_color = "yellow2",
            td_shape = 3,
-           td_size = 1) {
+           td_size = 1,
+           td_alpha = 1) {
     stopifnot(
       is.numeric(div_shape) || is.character(div_shape),
       is.numeric(td_shape) || is.character(td_shape),
       is.character(div_color),
       is.character(td_color),
       is.numeric(div_size),
-      is.numeric(td_size)
+      is.numeric(td_size),
+      is.numeric(div_alpha) && div_alpha >= 0 && div_alpha <= 1,
+      is.numeric(td_alpha) && td_alpha >= 0 && td_alpha <= 1
     )
     style <- list(
       color = c(div = div_color, td = td_color),
       shape = c(div = div_shape, td = td_shape),
-      size = c(div = div_size, td = td_size)
+      size = c(div = div_size, td = td_size),
+      alpha = c(div = div_alpha, td = td_alpha)
     )
-    structure(style, class = c(class(style), "pairs_style_np"))
+    structure(style, class = c(class(style), "nuts_style"))
   }
 
+
 #' @rdname MCMC-scatterplots
 #' @export
 #' @param chains,draws,nuts Optional arguments to the \code{pairs_condition}
@@ -569,27 +623,50 @@ pairs_condition <- function(chains = NULL, draws = NULL, nuts = NULL) {
 
 
 # internal ----------------------------------------------------------------
+#' @importFrom dplyr pull
 .mcmc_scatter <- function(x,
-                         pars = character(),
-                         regex_pars = character(),
-                         transformations = list(),
-                         hex = FALSE,
-                         size = 2.5,
-                         alpha = 0.8,
-                         binwidth = NULL) {
+                          pars = character(),
+                          regex_pars = character(),
+                          transformations = list(),
+                          hex = FALSE,
+                          size = 2.5,
+                          alpha = 0.8,
+                          binwidth = NULL,
+                          np = NULL,
+                          np_style = scatter_style_np()) {
   x <- prepare_mcmc_array(x, pars, regex_pars, transformations)
-  if (num_params(x) != 2)
+  if (num_params(x) != 2) {
     stop(
       "For 'mcmc_scatter' and 'mcmc_hex' exactly 2 parameters must be selected. ",
       "'mcmc_pairs' can be used for more than 2 parameters."
     )
+  }
 
   x <- merge_chains(x)
   parnames <- colnames(x)[1:2]
-  graph <- ggplot(
-    data = data.frame(x = c(x[, 1]), y = c(x[, 2])),
-    mapping = aes_(x = ~ x, y = ~ y)
-  )
+  has_divs <- !is.null(np)
+
+  xydata <- data.frame(x = c(x[, 1]), y = c(x[, 2]))
+  if (has_divs) {
+    if (hex) {
+      warning("'np' is currently ignored for hex plots.")
+    }
+    stopifnot(inherits(np_style, "nuts_style"))
+    np <- validate_nuts_data_frame(np)
+    param <- sym("Parameter")
+    val <- sym("Value")
+    divg <- sym("Divergent")
+    xydata$Divergent <-
+      np %>%
+      dplyr::filter(UQ(param) == "divergent__") %>%
+      pull(UQ(val))
+
+    divdata <- dplyr::filter(xydata, UQ(divg) == 1)
+    xydata <- dplyr::filter(xydata, UQ(divg) == 0)
+  }
+
+  graph <- ggplot(data = xydata, aes_(x = ~ x, y = ~ y))
+
   if (!hex) { # scatterplot
     graph <- graph +
       geom_point(
@@ -599,6 +676,17 @@ pairs_condition <- function(chains = NULL, draws = NULL, nuts = NULL) {
         size = size,
         alpha = alpha
       )
+
+    if (has_divs) {
+      graph <- graph +
+        geom_point(
+          data = divdata,
+          color = np_style$color[["div"]],
+          size = np_style$size[["div"]],
+          alpha = np_style$alpha[["div"]],
+          shape = np_style$shape[["div"]]
+        )
+    }
   } else { # hex binning
     suggested_package("scales")
     graph <- graph +
@@ -770,7 +858,8 @@ handle_condition <- function(x, condition=NULL, np=NULL, lp=NULL) {
       mark <- unstack_to_matrix(lp, Value ~ Chain)
 
     } else {
-      mark <- filter_(np, ~ Parameter == condition)
+      param <- sym("Parameter")
+      mark <- dplyr::filter(np, UQ(param) == condition)
       mark <- unstack_to_matrix(mark, Value ~ Chain)
     }
     if (condition == "divergent__") {
